

/*******************************************************************************

  Timer Programvel

  Com PIC16F628A  e display LCD  16 x 2


  Autor: Cludio Lrios

  Data: 10/08/2015

  Clock: 4mhz
  (metodo acumulo de erro)

  Temporizador programvel mx. ligado = 59 minutos e 59 segundos
  Apresenta mensagem: 'Timer On','Timer Off','Timer Pause'
  Produz beep ao termino da temporizao para alertar o usurio

  Use XC8 compiler
  Obs. Arquivo para fins didticos apenas.
  Este arquivo  parte integrante do blog Lrios.Tecnologia.WS

*******************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h> //para usar funoes de string deve se adicionar este header
#include <xc.h>
#define _XTAL_FREQ 4000000//frequncia usada (xtal externo))

// CONFIG
#pragma config FOSC = HS        // Oscillator Selection bits (HS oscillator: High-speed crystal/resonator on RA6/OSC2/CLKOUT and RA7/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = ON       // RA5/MCLR/VPP Pin Function Select bit (RA5/MCLR/VPP pin function is MCLR)
#pragma config BOREN = ON       // Brown-out Detect Enable bit (BOD enabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable bit (RB4/PGM pin has digital I/O function, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EE Memory Code Protection bit (Data memory code protection off)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)


//bytes
#define trisb TRISB
#define trisa TRISA
#define portb PORTB
#define porta PORTA

// lcd
#define   en   RB2 //rb2  lcd 'enable' pino 8
#define   rs   RB3 //rb3  lcd 'rs' - comando/dado pino 9
#define   db4  RB4 //rb4  db4 pino 10
#define   db5  RB5 //rb5  db5 pino 11
#define   db6  RB6 //rb6  db6 pino 12
#define   db7  RB7 //rb7  db7 pino 13
#define   tris_en   TRISB2 //rb2  lcd 'enable' pino 8
#define   tris_rs   TRISB3 //rb3  lcd 'rs' - comando/dado pino 9
#define   tris_db4  TRISB4 //rb4  dir db4 pino 10
#define   tris_db5  TRISB5 //rb5  dir db5 pino 11
#define   tris_db6  TRISB6 //rb6  dir db6 pino 12
#define   tris_db7  TRISB7 //rb7  dir db7 pino 13

//bits
#define rele    RA3 //rele pino 3
#define sw_min  RA2  //ajuste dos minutos
#define sw_stp  RA1  //boto de start-pause
#define sw_seg  RA0  //ajustes dos segundos
#define beep    RB1  //saida do beep (piezoeletrico simples)

//variveis globais
long cnt32=0;
char cnt=0;
char minutos=0;
char segundos=0;
bit f_trava1=0;
bit f_bipar=0;
bit f_dl=0;
bit f_pause=0;

//==============================================================================
//                   ROTINAS PARA LCD
//==============================================================================
//clock para lcd
void lcd_clk(void){
 en=1;
 __delay_us(40);//conforme o Lcd pode ser menor
 en=0;

 }

//envia nible para lcd
 void env_nib(char b){

 if(b&0x10)db4=1; else db4=0;
 if(b&0x20)db5=1; else db5=0;
 if(b&0x40)db6=1; else db6=0;
 if(b&0x80)db7=1; else db7=0;
 lcd_clk();
 }

 //envia comando para lcd
 void env_lcd_c(char val){
 rs=0;
 env_nib(val);
 env_nib(val<<4);
 __delay_ms(10);
}

//envia dados para lcd
 void env_lcd_d(char val){
 rs=1;
 env_nib(val);
  env_nib(val<<4);
 }

 //envia string de caracteres para lcd
 void Print_const(const char *val){
  
  char index=0;
  char nchar=strlen(val);//tamanho da string de caracteres
    rs=1;
    while(nchar--){
      env_nib(val[index]);
      env_nib(val[index++]<<4);
    }
 }
//inicializao do lcd
void lcd_init(void){
 char a;

  rs=0;
  tris_en=0;//direo dos pinos como saida
  tris_rs=0;
  tris_db4=0;
  tris_db5=0;
  tris_db6=0;
  tris_db7=0;
  db4=0;//situao inicial resetados
  db5=0;
  db6=0;
  db7=0;
  en=0;
  __delay_ms(50);//delay inicial
  db4=1; db5=1;// setados
  for(a=0;a<3;a++) {lcd_clk(); __delay_ms(5);}// envia 3,3,3
  db4=0;//envia 2
  lcd_clk();//pulso em 'en'
  __delay_ms(5);
  env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
  env_lcd_c(0x06);// no shift
  env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
  env_lcd_c(0x01);//limpa tela
  __delay_ms(10);

}
//-------------------------FIM DAS ROTINAS DE LCD--------------------------------


//rotina de produo de som no disco piezoeletrico
 void bipar() {
   int c;
   char a;
   f_bipar=0;
   for(a=0;a<5;a++){
    for(c=0; c<2500; c++) {

      beep=1;
      __delay_us(193); //ajuste conforme freq. ressonancia do piezoeletrico
      beep=0;
      __delay_us(11);
    }
    __delay_ms(100);//delay entre beeps
  }
 }


 //=============================================================================
 //         ROTINA DE INTERRUPO DO TIMER 0  
 //=============================================================================

void interrupt timer0_isr()
{
//--------------------------------------------------------
//gera 1 segundos com metodo de 'acumulo de erro' e pode ser usado
// para cristais com valores quebrados das mais variadas frequencias

 #define presc  64 // valor do prescaller do timer usado (8bits)
 #define fator  (long) 256*presc*4
 
   T0IF=0;//reseta flag de interrupo
   cnt32+=fator;
   if(cnt32>=_XTAL_FREQ){ cnt32-=_XTAL_FREQ;
   //obs. este metodo gasta mais linhas de cdigo 
 //-------------------------------------------------------         
          
         if(--segundos==0xff){
           segundos=59;
           if( minutos>0) --minutos;
           }
         if(!minutos&&!segundos) {
               rele=0;//desliga o rele de sada
               T0IE=0;//para a contagem
               f_bipar=1;//sinaliza para bipar (fim da contagem)
               f_pause=0;
        }
    }
    
    if(++cnt==64)cnt=0; //apenas para piscar os dois pontos ':'
}

//=============================================================================
 //    ROTINA DE MAIN
 //=============================================================================


void main() {

   OPTION_REG=5;//prescaller do timer 0 divide por 64 
   CMCON=7;//desliga comparadores internos
   PEIE=1;//liga chave geral de perifricos
   GIE=1;//liga chave geral das interrupes
   trisb = 0;//Define sentido do portb como saida
   trisa = 0b11100111;//Define sentido do porta in/out
   portb=0;//reseta portb
   rele= 0; //desativa rele de saida e desliga a carga
   lcd_init(); //Inicializa o display
   env_lcd_c(0x80);//linha 1
   Print_const("Timer Eletronico");// nome do aparelho
   env_lcd_c(0xc0);//linha 2
   Print_const("Larios.Tecnol.Ws");//blog
   __delay_ms(2000);//tempo de apresentao
   env_lcd_c(1);//limpa tela

 //=============================================================================
 //   LAO PRINCIPAL DE REPETIO
 //=============================================================================

   for(;;){


       if(!rele){ //ajustes somente em 'pausa'
       if(sw_seg){ // boto de ajuste de segundos se pressionado comea incrementar
         if(++segundos>59) segundos=0;
         __delay_ms(250);//velocidade de incremento
         if(!sw_seg) if(--segundos==0xff) segundos=59;;//se boto for solto durante o delay acima
         f_pause=0;//muda de 'pause' para 'off'
        }


        if(sw_min){ //boto ajuste de minutos se pressionado comea incrementar
           if(++minutos>59) minutos=0;
            __delay_ms(250);
            if(!sw_min) if(--minutos==0xff) minutos=59;//se boto for solto durante o delay acima
            f_pause=0;//muda de "pause' para 'off'
         }

       }

      if(sw_stp&&!f_trava1) {//boto start/pause  (1 toque 'liga' outro 'pausa')
         f_trava1=1;
        if((segundos>0)||(minutos>0)){//liga menos com '00:00'
         if(!rele){//liga a contagem
            rele=1;//liga rele e aciona a carga
            f_pause=1;
            T0IE=1;//liga interrupo do tmr0
            }
         else {
           T0IE=0;//desliga interrupo do tmr0//para a contagem
           rele=0; //desliga na pausa
         }
       }
      }
      if(!sw_stp) f_trava1=0; //reseta a trava

      env_lcd_c(0x83);//linha 1 p3
      if(T0IE){
      Print_const("Timer On   ");//imprime mensagem 'Timer on '
        f_dl=0;
      }
      else{
        if(!f_pause){
      Print_const("Timer Off  ");//imprime mensagem 'Timer on'
        f_dl=1;//autoriza o bip somente aps imprimir o acima
        }
        else{
      Print_const("Timer Pause");//imprime mensagem 'Timer on '
           f_dl=0;
        }
      }


      env_lcd_c(0xc5);//linha 2 p5
      env_lcd_d(minutos/10+'0');//envia dezenas de minutos
      env_lcd_d(minutos%10+'0');//envia unidades de minutos
      if( T0IE && (cnt<20)) //pisca ':' muito brevemente
          { env_lcd_d(' ');}//apaga ':'
      else
        { env_lcd_d(':');}//mostra ':'
      env_lcd_d(segundos/10+'0');//envia dezenas de segundos
      env_lcd_d(segundos%10+'0');//envia unidades de segundos
      if((f_bipar)&&(f_dl)) {f_dl=0; bipar();} //beep de fim da temporizao
      __delay_ms(1);//delay do loop principal
   }


}// for(;;)


//======================Fim do Programa=========================================









